package org.nakedobjects.nof.reflect.java.collection;

import java.util.Enumeration;

import org.nakedobjects.noa.spec.NakedObjectSpecification;
import org.nakedobjects.nof.testsystem.ProxyTestCase;
import org.nakedobjects.nof.testsystem.TestPojo;
import org.nakedobjects.nof.testsystem.TestProxyNakedObject;

public class ArrayAdapterTest extends ProxyTestCase {

    private ArrayAdapter arrayAdapter;
    private TestProxyNakedObject item0;
    private TestProxyNakedObject item1;
    private TestProxyNakedObject item2;

    protected void setUp() throws Exception {
        super.setUp();

        NakedObjectSpecification spec = system.getSpecification(TestPojo.class);
        TestPojo[] array = new TestPojo[3];
        arrayAdapter = new ArrayAdapter(array, spec);
        
        item0 = system.createPersistentTestObject();
        item1 = system.createPersistentTestObject();
        item2 = system.createPersistentTestObject();
        
        array[0] = (TestPojo) item0.getObject();
        array[1] = (TestPojo) item1.getObject();
        array[2] = (TestPojo) item2.getObject();
    }
    
    public void testSize() throws Exception {
        assertEquals(3, arrayAdapter.size());
    }
    
    public void testContains() throws Exception {
        assertTrue(arrayAdapter.contains(item1));
    }
    
    public void testContainsCantFindElement() throws Exception {
        TestProxyNakedObject unknownObject = system.createPersistentTestObject();
        assertFalse(arrayAdapter.contains(unknownObject));
    }
    
    public void testFirstObject() {
        assertEquals(item0, arrayAdapter.firstElement());
    }
    
    public void testFirstElementWhenNoArrayExist() {
        arrayAdapter = new ArrayAdapter(new TestPojo[0], null);
        assertEquals(null, arrayAdapter.firstElement());
    }
    
    public void testElements() throws Exception {
        Enumeration elements = arrayAdapter.elements();
        assertTrue(elements.hasMoreElements());
        assertEquals(item0, elements.nextElement());
        elements.nextElement();
        elements.nextElement();
        assertFalse(elements.hasMoreElements());
    }
    
    public void testInitCopiesArray() throws Exception {
        TestPojo[] toCopy = new TestPojo[] { new TestPojo(), new TestPojo() };
        arrayAdapter.init(toCopy);
        
        assertEquals(2, arrayAdapter.size());
        Object[] objects = ((Object[]) arrayAdapter.getObject());
        assertEquals(toCopy[0], objects[0]);
        assertEquals(toCopy[1], objects[1]);
    }
}


// Copyright (c) Naked Objects Group Ltd.
