package org.nakedobjects.nof.reflect.java.reflect;

import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.log4j.Level;
import org.apache.log4j.LogManager;
import org.nakedobjects.noa.annotations.MemberOrderAnnotation;
import org.nakedobjects.nof.reflect.peer.MemberPeer;


public class DeweyOrderSetTest extends TestCase {

    public static void main(final String[] args) {
        junit.textui.TestRunner.run(new TestSuite(DeweyOrderSetTest.class));
    }


    private MemberPeerStub lastNameMember = new MemberPeerStub("LastName");
    private MemberPeerStub firstNameMember = new MemberPeerStub("FirstName");
    private MemberPeerStub houseNumberMember = new MemberPeerStub("HouseNumber");
    private MemberPeerStub streetNameMember = new MemberPeerStub("StreetName");
    private MemberPeerStub postalTownMember = new MemberPeerStub("PostalTown");
    private MemberPeer[] lastNameAndFirstName = new MemberPeerStub[] {lastNameMember, firstNameMember};
    private MemberPeer[] nameAndAddressMembers = new MemberPeerStub[] {lastNameMember, firstNameMember, houseNumberMember, streetNameMember, postalTownMember};
    private MemberPeer[] lastNameFirstNameAndPostalTown = new MemberPeerStub[] {lastNameMember, firstNameMember, postalTownMember};
    
    
    protected void setUp() {
        LogManager.getLoggerRepository().setThreshold(Level.OFF);
        
        
    }

    protected void tearDown() throws Exception {
    }

    public void testDefaultGroup() {
        lastNameMember.addExtension(new MemberOrderAnnotation("", "1"));
        firstNameMember.addExtension(new MemberOrderAnnotation("", "2"));
        
        DeweyOrderSet orderSet = DeweyOrderSet.createOrderSet(lastNameAndFirstName);
        assertEquals("", orderSet.getGroupName());
        assertEquals("", orderSet.getGroupFullName());
        assertEquals("", orderSet.getGroupPath());
    }

    public void testDefaultGroupSize() {
        lastNameMember.addExtension(new MemberOrderAnnotation("", "1"));
        firstNameMember.addExtension(new MemberOrderAnnotation("", "2"));
        
        DeweyOrderSet orderSet = DeweyOrderSet.createOrderSet(lastNameAndFirstName);
        assertEquals(2, orderSet.size());
        assertEquals(2, orderSet.elementList().size());
        assertEquals(0, orderSet.children().size());
    }

    public void testDefaultGroupTwoMembersSorted() {
        lastNameMember.addExtension(new MemberOrderAnnotation("", "1"));
        firstNameMember.addExtension(new MemberOrderAnnotation("", "2"));
        
        DeweyOrderSet orderSet = DeweyOrderSet.createOrderSet(lastNameAndFirstName);
        assertEquals(lastNameMember, orderSet.elementList().get(0));
        assertEquals(firstNameMember, orderSet.elementList().get(1));
    }

    public void testTwoMembersAtDefaultGroupOtherWay() {
        lastNameMember.addExtension(new MemberOrderAnnotation("", "2"));
        firstNameMember.addExtension(new MemberOrderAnnotation("", "1"));
        
        DeweyOrderSet orderSet = DeweyOrderSet.createOrderSet(lastNameAndFirstName);
        assertEquals(firstNameMember, orderSet.elementList().get(0));
        assertEquals(lastNameMember, orderSet.elementList().get(1));
    }


    public void testWithChildGroupDefaultGroupName() {
        lastNameMember.addExtension(new MemberOrderAnnotation("", "1"));
        firstNameMember.addExtension(new MemberOrderAnnotation("", "2"));
        houseNumberMember.addExtension(new MemberOrderAnnotation("address", "1"));
        streetNameMember.addExtension(new MemberOrderAnnotation("address", "2"));
        postalTownMember.addExtension(new MemberOrderAnnotation("address", "3"));
        
        DeweyOrderSet orderSet = DeweyOrderSet.createOrderSet(nameAndAddressMembers);
        assertEquals("", orderSet.getGroupName());
        assertEquals("", orderSet.getGroupFullName());
        assertEquals("", orderSet.getGroupPath());
    }

    public void testWithChildGroupSize() {
        lastNameMember.addExtension(new MemberOrderAnnotation("", "1"));
        firstNameMember.addExtension(new MemberOrderAnnotation("", "2"));
        houseNumberMember.addExtension(new MemberOrderAnnotation("address", "1"));
        streetNameMember.addExtension(new MemberOrderAnnotation("address", "2"));
        postalTownMember.addExtension(new MemberOrderAnnotation("address", "3"));
        
        DeweyOrderSet orderSet = DeweyOrderSet.createOrderSet(nameAndAddressMembers);
        assertEquals(1, orderSet.children().size());
        assertEquals(3, orderSet.size());
    }

    public void testWithChildGroupChildsGroupName() {
        lastNameMember.addExtension(new MemberOrderAnnotation("", "1"));
        firstNameMember.addExtension(new MemberOrderAnnotation("", "2"));
        houseNumberMember.addExtension(new MemberOrderAnnotation("address", "1"));
        streetNameMember.addExtension(new MemberOrderAnnotation("address", "2"));
        postalTownMember.addExtension(new MemberOrderAnnotation("address", "3"));
        
        DeweyOrderSet orderSet = DeweyOrderSet.createOrderSet(nameAndAddressMembers);
        OrderSet childOrderSet = (OrderSet) orderSet.children().get(0);
        assertEquals("Address", childOrderSet.getGroupName());
        assertEquals("address", childOrderSet.getGroupFullName());
        assertEquals("", childOrderSet.getGroupPath());
    }


    public void testWithChildGroupChildsGroupSize() {
        lastNameMember.addExtension(new MemberOrderAnnotation("", "1"));
        firstNameMember.addExtension(new MemberOrderAnnotation("", "2"));
        houseNumberMember.addExtension(new MemberOrderAnnotation("address", "1"));
        streetNameMember.addExtension(new MemberOrderAnnotation("address", "2"));
        postalTownMember.addExtension(new MemberOrderAnnotation("address", "3"));
        
        DeweyOrderSet orderSet = DeweyOrderSet.createOrderSet(nameAndAddressMembers);
        OrderSet childOrderSet = (OrderSet) orderSet.children().get(0);
        assertEquals(3, childOrderSet.size());
        assertEquals(0, childOrderSet.children().size());
    }


    public void testWithChildGroupChildsGroupElementOrdering() {
        lastNameMember.addExtension(new MemberOrderAnnotation("", "1"));
        firstNameMember.addExtension(new MemberOrderAnnotation("", "2"));
        houseNumberMember.addExtension(new MemberOrderAnnotation("address", "6"));
        streetNameMember.addExtension(new MemberOrderAnnotation("address", "5"));
        postalTownMember.addExtension(new MemberOrderAnnotation("address", "4"));
        
        DeweyOrderSet orderSet = DeweyOrderSet.createOrderSet(nameAndAddressMembers);
        OrderSet childOrderSet = (OrderSet) orderSet.children().get(0);
        assertEquals(postalTownMember, childOrderSet.elementList().get(0));
        assertEquals(streetNameMember, childOrderSet.elementList().get(1));
        assertEquals(houseNumberMember, childOrderSet.elementList().get(2));
    }

    public void testWithChildGroupOrderedAtEnd() {
        houseNumberMember.addExtension(new MemberOrderAnnotation("address", "6"));
        streetNameMember.addExtension(new MemberOrderAnnotation("address", "5"));
        postalTownMember.addExtension(new MemberOrderAnnotation("address", "4"));
        lastNameMember.addExtension(new MemberOrderAnnotation("", "3"));
        firstNameMember.addExtension(new MemberOrderAnnotation("", "2"));
        
        DeweyOrderSet orderSet = DeweyOrderSet.createOrderSet(nameAndAddressMembers);
        assertEquals(firstNameMember, orderSet.elementList().get(0));
        assertEquals(lastNameMember, orderSet.elementList().get(1));
        assertTrue(orderSet.elementList().get(2) instanceof OrderSet);
    }

    
    public void testDefaultGroupNeitherAnnotatedSize() {
        DeweyOrderSet orderSet = DeweyOrderSet.createOrderSet(lastNameAndFirstName);
        assertEquals(2, orderSet.elementList().size());
    }

    public void testDefaultGroupNeitherAnnotatedOrderedByName() {
        DeweyOrderSet orderSet = DeweyOrderSet.createOrderSet(lastNameAndFirstName);
        assertEquals(firstNameMember, orderSet.elementList().get(0));
        assertEquals(lastNameMember, orderSet.elementList().get(1));
    }


    public void testDefaultGroupMixOfAnnotatedAndNotSize() {
        lastNameMember.addExtension(new MemberOrderAnnotation("", "1"));
        postalTownMember.addExtension(new MemberOrderAnnotation("address", "2"));
        
        DeweyOrderSet orderSet = DeweyOrderSet.createOrderSet(lastNameFirstNameAndPostalTown);
        assertEquals(3, orderSet.elementList().size());
    }

    public void testDefaultGroupMixOfAnnotatedAndNotOrderedWithAnnotatedFirst() {
        lastNameMember.addExtension(new MemberOrderAnnotation("", "1"));
        postalTownMember.addExtension(new MemberOrderAnnotation("", "2"));
        
        DeweyOrderSet orderSet = DeweyOrderSet.createOrderSet(lastNameFirstNameAndPostalTown);
        
        assertEquals(lastNameMember, orderSet.elementList().get(0));
        assertEquals(postalTownMember, orderSet.elementList().get(1));
        assertEquals(firstNameMember, orderSet.elementList().get(2));
    }

}
// Copyright (c) Naked Objects Group Ltd.
