package org.nakedobjects.nof.reflect.java.reflect;

import java.lang.reflect.Method;

import junit.framework.TestCase;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.nakedobjects.noa.adapter.Naked;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.reflect.Consent;
import org.nakedobjects.noa.reflect.NakedObjectAction;
import org.nakedobjects.noa.spec.NakedObjectSpecification;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nof.reflect.peer.MemberIdentifierImpl;
import org.nakedobjects.testing.TestSpecification;
import org.nakedobjects.testing.TestSystem;


public class JavaActionTest extends TestCase {
    public static void main(final String[] args) {
        junit.textui.TestRunner.run(JavaActionTest.class);
    }

    private TestSystem system;
    private JavaAction javaAction;
    private NakedObject nakedObject;
    private JavaActionTestObject javaObject;

    protected void setUp() throws Exception {
        super.setUp();
        Logger.getRootLogger().setLevel(Level.OFF);

        system = new TestSystem();
        system.init();

        system.addSpecificationToLoader(new TestSpecification(JavaActionTestObject.class));

        javaObject = new JavaActionTestObject();
        nakedObject = NakedObjectsContext.getObjectLoader().createAdapterForTransient(javaObject, false);

        Class cls = Class.forName(getClass().getName() + "Object");
        Method action = cls.getDeclaredMethod("actionMethod", new Class[0]);

        Method visible = cls.getDeclaredMethod("invisibleMethod", new Class[0]);
        Method valid = cls.getDeclaredMethod("validMethod", new Class[0]);

        Method labelMethod = cls.getDeclaredMethod("labelMethod", new Class[] { String.class });
        Method mandatoryMethod = cls.getDeclaredMethod("mandatoryMethod", new Class[] { String.class });

        ParameterMethods parameterMethods = new ParameterMethods(new MethodHelper(labelMethod),
                new MethodHelper(mandatoryMethod), null, null, null, null, null, null, null);
        GeneralControlMethods controlMethods = new GeneralControlMethods(new MethodHelper(visible), null, valid, null);
        DescriptiveMethods descriptiveMethods = new DescriptiveMethods(new StaticHelper("name"), null);
        MemberSessionMethods sessionMethods = new MemberSessionMethods(null, null);

        javaAction = new JavaAction(new MemberIdentifierImpl("cls", "methodName", (NakedObjectSpecification[])null), NakedObjectAction.EXPLORATION,
                NakedObjectAction.LOCAL, null, new NakedObjectSpecification[0], action, null,  parameterMethods, descriptiveMethods,
                controlMethods, new ActionFlags(), sessionMethods, null);
        assertNotNull(javaAction);
    }

    protected void tearDown() throws Exception {
        system.shutdown();
    }

    public void testAction() throws Exception {
        TestSpecification spec = new TestSpecification();
        system.addSpecificationToLoader(spec);

        javaAction.execute(nakedObject, new Naked[0]);
    }

    public void testMethodName() throws Exception {
        assertEquals(new MemberIdentifierImpl("cls", "methodName"), javaAction.getIdentifier());
    }

    public void testReturnType() {
        assertNull(javaAction.getReturnType());
    }

    public void testType() {
        assertEquals(NakedObjectAction.EXPLORATION, javaAction.getType());
    }

    public void testTarget() {
        assertEquals(NakedObjectAction.LOCAL, javaAction.getTarget());
    }

    public void testInvisible() {
        assertFalse("invisible", javaAction.isVisible(nakedObject));
    }

    /*
     * public void testParameterLabels() { ActionParameterSet parameterSet =
     * javaAction.createParameterSet(nakedObject, new Naked[0]); assertEquals("label",
     * parameterSet.getParameterLabels()[0]); }
     */
    public void testParameterSetValid() throws Exception {
        Consent consent = javaAction.isParameterSetValidImperatively(nakedObject, new Naked[0]);
        assertEquals(false, consent.isAllowed());
        assertEquals("invalid", consent.getReason());
    }
}
// Copyright (c) Naked Objects Group Ltd.
