package org.nakedobjects.nof.reflect.java.reflect;

import java.lang.reflect.Method;

import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.annotations.When;
import org.nakedobjects.nof.reflect.peer.MemberIdentifierImpl;
import org.nakedobjects.testing.TestSpecification;
import org.nakedobjects.testing.TestSystem;



public class JavaAssociationTest extends TestCase {
    private static final String PERSON_FIELD_NAME = "person";
    private JavaObjectWithOneToOneAssociations javaObjectWithOneToOneAssociations;
    private NakedObject nakedObjectHoldingObjectWithAssociations;
    private JavaOneToOneAssociation personField;
    private JavaReferencedObject javaObjectForReferencing;
    private NakedObject associate;
    private TestSpecification spec;
    private TestSystem system;

    public static void main(final String[] args) {
        junit.textui.TestRunner.run(new TestSuite(JavaAssociationTest.class));
    }

    protected void setUp() throws Exception {
        Logger.getRootLogger().setLevel(Level.OFF);

        system = new TestSystem();
        system.init();
        system.addSpecificationToLoader(new TestSpecification()); // for String
        system.addSpecificationToLoader(new TestSpecification()); // for Date
        system.addSpecificationToLoader(new TestSpecification()); // for float

        system.addSpecificationToLoader(new TestSpecification(JavaObjectWithOneToOneAssociations.class.getName()));
        system.addSpecificationToLoader(spec = new TestSpecification(JavaReferencedObject.class.getName()));

        javaObjectWithOneToOneAssociations = new JavaObjectWithOneToOneAssociations();
        nakedObjectHoldingObjectWithAssociations = system.createAdapterForTransient(javaObjectWithOneToOneAssociations);

        Class cls = JavaObjectWithOneToOneAssociations.class;
        Method get = cls.getDeclaredMethod("getReferencedObject", new Class[0]);
        Method set = cls.getDeclaredMethod("setReferencedObject", new Class[] { JavaReferencedObject.class });

        FieldMethods fieldMethods = new FieldMethods(get, set);
        GeneralControlMethods controlMethods = new GeneralControlMethods(null, null, null, null);
        DescriptiveMethods descriptiveMethods = new DescriptiveMethods(null, null);
        FieldFlags fieldFlags = new FieldFlags(When.NEVER, false, When.NEVER, false, "");
        FieldSessionMethods sessionMethods = new FieldSessionMethods(null, null);
        
        personField = new JavaOneToOneAssociation(new MemberIdentifierImpl("cls", PERSON_FIELD_NAME),
                JavaReferencedObject.class, descriptiveMethods, fieldMethods, controlMethods, fieldFlags, sessionMethods);

        javaObjectForReferencing = new JavaReferencedObject();
        associate = system.createAdapterForTransient(javaObjectForReferencing);
    }

    protected void tearDown() throws Exception {
        system.shutdown();
    }

    public void testType() {
        assertEquals(spec, personField.getSpecification());
    }

    public void testSet() {
        system.addSpecificationToLoader(new TestSpecification()); // for one-to-one
        system.addSpecificationToLoader(new TestSpecification()); // for object

        assertNull(javaObjectWithOneToOneAssociations.getReferencedObject());
        personField.setAssociation(nakedObjectHoldingObjectWithAssociations, associate);
        assertEquals(javaObjectForReferencing, javaObjectWithOneToOneAssociations.getReferencedObject());
    }

    public void testRemove() {
        system.addSpecificationToLoader(new TestSpecification()); // for one-to-one
        system.addSpecificationToLoader(new TestSpecification()); // for object

        javaObjectWithOneToOneAssociations.setReferencedObject(javaObjectForReferencing);

        assertNotNull(javaObjectWithOneToOneAssociations.getReferencedObject());
        personField.clearAssociation(nakedObjectHoldingObjectWithAssociations, associate);
        assertNull(javaObjectWithOneToOneAssociations.getReferencedObject());
    }

    public void testGet() {
        assertNull(personField.getAssociation(nakedObjectHoldingObjectWithAssociations));
        javaObjectWithOneToOneAssociations.setReferencedObject(javaObjectForReferencing);
        assertEquals(associate, personField.getAssociation(nakedObjectHoldingObjectWithAssociations));
    }

    public void testInitGet() {
        system.addSpecificationToLoader(new TestSpecification()); // for object
        system.addSpecificationToLoader(new TestSpecification()); // for object

        assertNull(javaObjectWithOneToOneAssociations.getReferencedObject());
        personField.initAssociation(nakedObjectHoldingObjectWithAssociations, associate);
        assertEquals(javaObjectForReferencing, javaObjectWithOneToOneAssociations.getReferencedObject());
    }

    public void testName() {
        assertEquals(new MemberIdentifierImpl("cls", PERSON_FIELD_NAME), personField.getIdentifier());
    }
}
// Copyright (c) Naked Objects Group Ltd.
