package org.nakedobjects.nof.reflect.java.reflect;

import java.lang.reflect.Method;
import java.util.ArrayList;

import junit.framework.TestCase;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.nof.reflect.peer.MemberIdentifier;
import org.nakedobjects.nof.reflect.peer.MemberIdentifierImpl;
import org.nakedobjects.nof.reflect.peer.ReflectionException;
import org.nakedobjects.nof.testsystem.TestPojo;
import org.nakedobjects.nof.testsystem.TestProxyNakedObject;
import org.nakedobjects.nof.testsystem.TestProxySystem;


public class JavaCollectionInvocationExceptionTest extends TestCase {

    private TestProxySystem system;
    private JavaCollection listField;
    private TestObjectWithCollection collection;
    private TestProxyNakedObject collectionAdapter;
    private ArrayList arrayList;

    protected void setUp() throws Exception {
        Logger.getRootLogger().setLevel(Level.OFF);

        system = new TestProxySystem();
        system.init();

        Class cls = TestPojo.class;

        Method getMethod = TestObjectWithCollection.class.getMethod("getList", new Class[0]);
        Method isValidMethod = TestObjectWithCollection.class.getMethod("validateAddToList", new Class[] { Object.class });
        Method isRemoveMethod = TestObjectWithCollection.class.getMethod("validateAddToList", new Class[] { Object.class });
        Method addMethod = TestObjectWithCollection.class.getMethod("addToList", new Class[] { Object.class });
        Method removeMethod = TestObjectWithCollection.class.getMethod("removeFromList", new Class[] { Object.class });
        Method clearMethod = TestObjectWithCollection.class.getMethod("clearList", new Class[] {});

        MemberIdentifier identifer = new MemberIdentifierImpl(TestObjectWithCollection.class.getName());
        FieldMethods fieldMethods = new FieldMethods(getMethod, null, addMethod, removeMethod, null, null);
        FieldFlags fieldFlags = new FieldFlags(null, false, null, true, "");
        GeneralControlMethods controlMethods = new GeneralControlMethods(null, null, isValidMethod, isRemoveMethod);
        MemberHelper nameHelper = new StaticHelper("list");
        DescriptiveMethods descriptiveMethods = new DescriptiveMethods(nameHelper, null);
        MemberSessionMethods sessionMethods = new MemberSessionMethods(null, null);

        listField = new JavaCollection(identifer, cls, descriptiveMethods, fieldMethods, controlMethods, fieldFlags, clearMethod,
                sessionMethods);

        arrayList = new ArrayList();
        collection = new TestObjectWithCollection(arrayList, true);
        collectionAdapter = system.createPersistentTestObject(collection);
    }

    public void testGetAssociations() throws Exception {
        try {
            listField.getAssociations(collectionAdapter);
            fail();
        } catch (ReflectionException e) {}
    }

    public void testAdapterReflectsEmptyCollection() throws Exception {
        try {
            listField.isEmpty(collectionAdapter);
            fail();
        } catch (ReflectionException e) {}
    }

    public void testAddAssociation() {
        try {
            TestPojo elementToAdd = new TestPojo();
            listField.addAssociation(collectionAdapter, system.createPersistentTestObject(elementToAdd));
            fail();
        } catch (ReflectionException e) {}

    }

    public void testRemoveAssociation() {
        try {
            TestPojo elementToAdd = new TestPojo();
            TestProxyNakedObject elementAdapter = system.createPersistentTestObject(elementToAdd);
            listField.removeAssociation(collectionAdapter, elementAdapter);
            fail();
        } catch (ReflectionException e) {}

    }

    public void testInitAssociations() {
        try {
            TestPojo elementToAdd1 = new TestPojo();
            TestPojo elementToAdd2 = new TestPojo();
            listField.initOneToManyAssociation(collectionAdapter, new NakedObject[] {
                    system.createPersistentTestObject(elementToAdd1), system.createPersistentTestObject(elementToAdd2) });
            fail();
        } catch (ReflectionException e) {}
    }

    public void testRemoveAllAssociations() {
        try {
            listField.removeAllAssociations(collectionAdapter);
            assertEquals(0, arrayList.size());
            assertEquals("2", collectionAdapter.getVersion().sequence());
            fail();
        } catch (ReflectionException e) {}

    }

    public void testIsAddValid() throws Exception {
        try {
            TestPojo elementToAdd = new TestPojo();
            listField.isAddValid(collectionAdapter, system.createPersistentTestObject(elementToAdd));
            fail();
        } catch (ReflectionException e) {}

    }

    public void testIsRemoveValid() throws Exception {
        try {
            TestPojo elementToAdd = new TestPojo();
            listField.isRemoveValid(collectionAdapter, system.createPersistentTestObject(elementToAdd));
            fail();
        } catch (ReflectionException e) {}
    }

}

// Copyright (c) Naked Objects Group Ltd.
