package org.nakedobjects.nof.reflect.java.reflect;

import java.lang.reflect.Method;
import java.util.ArrayList;

import junit.framework.TestCase;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.reflect.Consent;
import org.nakedobjects.nof.core.reflect.Allow;
import org.nakedobjects.nof.core.util.DebugString;
import org.nakedobjects.nof.reflect.peer.MemberIdentifier;
import org.nakedobjects.nof.reflect.peer.MemberIdentifierImpl;
import org.nakedobjects.nof.testsystem.TestPojo;
import org.nakedobjects.nof.testsystem.TestProxyNakedObject;
import org.nakedobjects.nof.testsystem.TestProxySystem;

public class JavaCollectionTest extends TestCase {

    private TestProxySystem system;
    private JavaCollection listField;
    private TestObjectWithCollection testPojo;
    private TestProxyNakedObject testAdapter;
    private ArrayList collection;

    protected void setUp() throws Exception {
        Logger.getRootLogger().setLevel(Level.OFF);
        
        system = new TestProxySystem();
        system.init();
        
        
        Class cls = TestPojo.class;
        
        Method getMethod = TestObjectWithCollection.class.getMethod("getList", new Class[0]);
        
        MemberIdentifier identifer = new MemberIdentifierImpl(TestObjectWithCollection.class.getName());
        FieldMethods fieldMethods = new FieldMethods(getMethod, null);
        FieldFlags fieldFlags = new FieldFlags(null, false, null, true, "");
        GeneralControlMethods controlMethods = new GeneralControlMethods(null, null, null, null);
        MemberHelper nameHelper = new StaticHelper("list");
        DescriptiveMethods descriptiveMethods = new DescriptiveMethods(nameHelper, null);
        MemberSessionMethods sessionMethods = new MemberSessionMethods(null, null);
        
        listField = new JavaCollection(identifer, cls, descriptiveMethods , fieldMethods , controlMethods , fieldFlags, null, sessionMethods );
        
        collection = new ArrayList();
        testPojo = new TestObjectWithCollection(collection, false);
        testAdapter = system.createPersistentTestObject(testPojo);
    }
    
    public void testType() throws Exception {
        assertEquals(system.getSpecification(TestPojo.class), listField.getSpecification());
    }

    public void testGetAssociations() {
        ArrayList expectedArrayList = new ArrayList(1);
        expectedArrayList.add("Test element");

        collection.add("Test element");
       
        assertEquals(expectedArrayList, listField.getAssociations(testAdapter).getObject());
    }

    public void testIsCollectionEmpty() {
        assertEquals(0, collection.size());
    }
    
    public void testAdapterReflectsEmptyCollection() throws Exception {
        assertTrue(listField.isEmpty(testAdapter));
    }

    public void testAdapterReflectsNullCollection() throws Exception {
        testAdapter = system.createPersistentTestObject(new TestObjectWithCollection(null, false));
        assertTrue(listField.isEmpty(testAdapter));
    }
    
    public void testAddAssociation() {
        TestPojo elementToAdd = new TestPojo();
        listField.addAssociation(testAdapter, system.createPersistentTestObject(elementToAdd));
        assertEquals(1, collection.size());
        assertEquals(elementToAdd, collection.get(0));
        assertEquals("2", testAdapter.getVersion().sequence());
    }
        
    public void testRemoveAssociation() {
        TestPojo elementToAdd = new TestPojo();
        TestProxyNakedObject elementAdapter = system.createPersistentTestObject(elementToAdd);
        listField.addAssociation(testAdapter, elementAdapter);
        
        listField.removeAssociation(testAdapter, elementAdapter);
        assertEquals(0, collection.size());
        assertEquals("3", testAdapter.getVersion().sequence());
    }
    
    public void testInitAssociations() {
        TestPojo elementToAdd1 = new TestPojo();
        TestPojo elementToAdd2 = new TestPojo();
        listField.initOneToManyAssociation(testAdapter, new NakedObject[] { 
                system.createPersistentTestObject(elementToAdd1),
                system.createPersistentTestObject(elementToAdd2) });
        assertEquals(2, collection.size());
        assertEquals(elementToAdd1, collection.get(0));
        assertEquals(elementToAdd2, collection.get(1));
        assertEquals("1", testAdapter.getVersion().sequence());
  }

    public void testRemoveAllAssociations() {
        collection.add(new TestPojo());
        collection.add(new TestPojo());
        assertEquals(2, collection.size());
        assertEquals("1", testAdapter.getVersion().sequence());

        listField.removeAllAssociations(testAdapter);
        assertEquals(0, collection.size());
        assertEquals("2", testAdapter.getVersion().sequence());
  }

    public void testIsAddValid() throws Exception {
        TestPojo elementToAdd = new TestPojo();
        Consent addValid = listField.isAddValid(testAdapter, system.createPersistentTestObject(elementToAdd));
        assertEquals(Allow.DEFAULT, addValid);
    }
    
    public void testIsRemoveValid() throws Exception {
        TestPojo elementToAdd = new TestPojo();
        Consent addValid = listField.isRemoveValid(testAdapter, system.createPersistentTestObject(elementToAdd));
        assertEquals(Allow.DEFAULT, addValid);
    }
    
    public void testDebug() throws Exception {
        DebugString debug = new DebugString();
        listField.debugData(debug);
        assertTrue(debug.toString().length() > 0);
    }

    public void testToString() throws Exception {
        assertTrue(listField.toString().length() > 0);
    }

}


// Copyright (c) Naked Objects Group Ltd.
