package org.nakedobjects.nof.reflect.java.reflect;

import java.lang.reflect.Method;
import java.util.ArrayList;

import junit.framework.TestCase;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.nakedobjects.noa.reflect.Consent;
import org.nakedobjects.nof.core.reflect.Allow;
import org.nakedobjects.nof.core.util.DebugString;
import org.nakedobjects.nof.reflect.peer.MemberIdentifier;
import org.nakedobjects.nof.reflect.peer.MemberIdentifierImpl;
import org.nakedobjects.nof.testsystem.TestPojo;
import org.nakedobjects.nof.testsystem.TestProxyNakedObject;
import org.nakedobjects.nof.testsystem.TestProxySystem;

public class JavaCollectionWithAllMethodsTest extends TestCase {
    private TestProxySystem system;
    private JavaCollection listField;
    private TestObjectWithCollection collection;
    private TestProxyNakedObject collectionAdapter;

    protected void setUp() throws Exception {
        Logger.getRootLogger().setLevel(Level.OFF);
        
        system = new TestProxySystem();
        system.init();
        
        
        Class cls = TestPojo.class;
        
        Method getMethod = TestObjectWithCollection.class.getMethod("getList", new Class[0]);
        Method isValidMethod = TestObjectWithCollection.class.getMethod("validateAddToList", new Class[] {Object.class});
        Method isRemoveMethod = TestObjectWithCollection.class.getMethod("validateAddToList", new Class[] {Object.class});
        Method addMethod = TestObjectWithCollection.class.getMethod("addToList", new Class[] {Object.class});
        Method removeMethod = TestObjectWithCollection.class.getMethod("removeFromList", new Class[] {Object.class});
        Method clearMethod = TestObjectWithCollection.class.getMethod("clearList", new Class[] {});

        MemberIdentifier identifer = new MemberIdentifierImpl(TestObjectWithCollection.class.getName());
        FieldMethods fieldMethods = new FieldMethods(getMethod, null, addMethod, removeMethod, null, null);
        FieldFlags fieldFlags = new FieldFlags(null, false, null, true, "");
        GeneralControlMethods controlMethods = new GeneralControlMethods(null, null, isValidMethod, isRemoveMethod);
        MemberHelper nameHelper = new StaticHelper("list");
        DescriptiveMethods descriptiveMethods = new DescriptiveMethods(nameHelper, null);
        MemberSessionMethods sessionMethods = new MemberSessionMethods(null, null);
        
        listField = new JavaCollection(identifer, cls, descriptiveMethods , fieldMethods , controlMethods , fieldFlags, clearMethod, sessionMethods );
        
        collection = new TestObjectWithCollection(new ArrayList(), false);
        collectionAdapter = system.createPersistentTestObject(collection);
    }
    
    public void testAddAssociationUsesAddMethod() {
        TestPojo elementToAdd = new TestPojo();
        listField.addAssociation(collectionAdapter, system.createPersistentTestObject(elementToAdd));
        assertEquals(elementToAdd, collection.getAdded());
        assertEquals("1", collectionAdapter.getVersion().sequence());
    }
        
    public void testRemoveAssociationUsesRemoveMethod() {
        TestPojo elementToAdd = new TestPojo();
       TestProxyNakedObject elementAdapter = system.createPersistentTestObject(elementToAdd);
        
        listField.removeAssociation(collectionAdapter, elementAdapter);
        assertEquals(elementToAdd, collection.getRemoved());
        assertEquals("1", collectionAdapter.getVersion().sequence());
    }
    
    public void testRemoveAllAssociationsUsesClearMethod() {
        listField.removeAllAssociations(collectionAdapter);
        assertTrue(collection.getClear());
        assertEquals("1", collectionAdapter.getVersion().sequence());
    }

    public void testIsAddValidCallsIsValidMethod() throws Exception {
        TestPojo elementToAdd = new TestPojo();
        Consent addValid = listField.isAddValid(collectionAdapter, system.createPersistentTestObject(elementToAdd));
        assertEquals(Allow.DEFAULT, addValid);
    }
    
    public void testIsAddValidFailsCallsIsValidMethod() throws Exception {
        TestPojo elementToAdd = new TestObjectWithCollection(null, false);
        Consent addValid = listField.isAddValid(collectionAdapter, system.createPersistentTestObject(elementToAdd));
        assertTrue(addValid.isVetoed());
    }
    
    public void testIsRemoveValidCallsIsValidMethod() throws Exception {
        TestPojo elementToAdd = new TestPojo();
        Consent addValid = listField.isRemoveValid(collectionAdapter, system.createPersistentTestObject(elementToAdd));
        assertEquals(Allow.DEFAULT, addValid);
    }
    
    public void testIsRemoveValidFailsCallsIsValidMethod() throws Exception {
        TestPojo elementToAdd = new TestObjectWithCollection(null, false);
        Consent addValid = listField.isRemoveValid(collectionAdapter, system.createPersistentTestObject(elementToAdd));
        assertTrue(addValid.isVetoed());
    }
    

    public void testDebug() throws Exception {
        DebugString debug = new DebugString();
        listField.debugData(debug);
        assertTrue(debug.toString().length() > 0);
    }

    public void testToString() throws Exception {
        assertTrue(listField.toString().length() > 0);
    }

}


// Copyright (c) Naked Objects Group Ltd.
