package org.nakedobjects.nof.reflect.java.reflect;

import java.lang.reflect.Method;

import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.annotations.When;
import org.nakedobjects.nof.reflect.peer.MemberIdentifierImpl;
import org.nakedobjects.testing.TestSpecification;
import org.nakedobjects.testing.TestSystem;



public class JavaOneToOneAssociationTest extends TestCase {
    private static final String PERSON_FIELD_NAME = "person";
    private JavaObjectWithOneToOneAssociations objectWithOneToOneAssoications;
    private JavaOneToOneAssociation personField;
    private JavaReferencedObject referencedObject;
    private TestSystem system;
    private NakedObject nakedObject;
    private NakedObject associatedNakedObject;
    private TestSpecification spec;

    public static void main(final String[] args) {
        junit.textui.TestRunner.run(new TestSuite(JavaOneToOneAssociationTest.class));
    }

    protected void setUp() throws Exception {
        super.setUp();
        Logger.getRootLogger().setLevel(Level.OFF);

        system = new TestSystem();
        system.init();

        system.addSpecificationToLoader(new TestSpecification(JavaObjectWithOneToOneAssociations.class.getName()));
        spec = new TestSpecification(JavaReferencedObject.class.getName());
        system.addSpecificationToLoader(spec);

        objectWithOneToOneAssoications = new JavaObjectWithOneToOneAssociations();

        nakedObject = system.createAdapterForTransient(objectWithOneToOneAssoications);

        Class cls = JavaObjectWithOneToOneAssociations.class;
        Method get = cls.getDeclaredMethod("getReferencedObject", new Class[0]);
        Method set = cls.getDeclaredMethod("setReferencedObject", new Class[] { JavaReferencedObject.class });

        Method visible = cls.getDeclaredMethod("hideReferencedObject", new Class[] { JavaReferencedObject.class });
        Method available = cls.getDeclaredMethod("availableReferencedObject", new Class[] { JavaReferencedObject.class });
        Method valid = cls.getDeclaredMethod("validReferencedObject", new Class[] { JavaReferencedObject.class });
              
        FieldMethods fieldMethods = new FieldMethods(get, set);
        GeneralControlMethods controlMethods = new GeneralControlMethods(new MethodHelper(visible), new MethodHelper(available), valid, null);
        DescriptiveMethods descriptiveMethods = new DescriptiveMethods(new StaticHelper("name"), null);
        FieldFlags xxMethods = new FieldFlags(When.NEVER, false, When.ALWAYS, false, "");
        FieldSessionMethods sessionMethods = new FieldSessionMethods(null, null);
    
        personField = new JavaOneToOneAssociation(new MemberIdentifierImpl("cls", PERSON_FIELD_NAME),
                JavaReferencedObject.class, descriptiveMethods, fieldMethods, controlMethods, xxMethods, sessionMethods);

        referencedObject = new JavaReferencedObject();
        associatedNakedObject = system.createAdapterForTransient(referencedObject);
    }

    protected void tearDown() throws Exception {
        system.shutdown();
    }

    public void testType() {
        assertEquals(spec, personField.getSpecification());
    }

    public void testSet() {
         assertNull(objectWithOneToOneAssoications.getReferencedObject());
        personField.setAssociation(nakedObject, associatedNakedObject);
        assertEquals(referencedObject, objectWithOneToOneAssoications.getReferencedObject());
    }

    public void testRemove() {
         objectWithOneToOneAssoications.setReferencedObject(referencedObject);

        assertNotNull(objectWithOneToOneAssoications.getReferencedObject());
        personField.clearAssociation(nakedObject, associatedNakedObject);
        assertNull(objectWithOneToOneAssoications.getReferencedObject());
    }

    public void testGet() {
        assertNull(objectWithOneToOneAssoications.getReferencedObject());
        objectWithOneToOneAssoications.setReferencedObject(referencedObject);
        assertEquals(referencedObject, personField.getAssociation(nakedObject).getObject());
    }

    public void testInitGet() {
        assertNull(objectWithOneToOneAssoications.getReferencedObject());
        personField.initAssociation(nakedObject, associatedNakedObject);
        assertEquals(referencedObject, objectWithOneToOneAssoications.getReferencedObject());
    }

    public void testName() {
        assertEquals(new MemberIdentifierImpl("cls", PERSON_FIELD_NAME), personField.getIdentifier());
    }
    
    public void testVisible() {
        assertFalse("invisible", personField.isVisible(nakedObject));
        objectWithOneToOneAssoications.visible = true;
        assertTrue("visible", personField.isVisible(nakedObject));
    }

    public void testValid() {
        assertFalse("not valid", personField.isAssociationValid(nakedObject, null).isAllowed());
        objectWithOneToOneAssoications.valid = true;
        assertTrue("valid", personField.isAssociationValid(nakedObject, null).isAllowed());
    }

    public void testUsable() {
        assertFalse(personField.isUsable(nakedObject).isAllowed());
        assertEquals("Field not editable", personField.isUsableDeclaratively().getReason());
    }
}
// Copyright (c) Naked Objects Group Ltd.
