package org.nakedobjects.nof.reflect.java.reflect;

import junit.framework.TestCase;

import org.apache.log4j.Level;
import org.apache.log4j.LogManager;
import org.nakedobjects.applib.marker.NonPersistable;
import org.nakedobjects.noa.adapter.Persistable;
import org.nakedobjects.noa.spec.NakedObjectSpecification;
import org.nakedobjects.nof.reflect.spec.AbstractSpecification;
import org.nakedobjects.nof.reflect.spec.ReflectionPeerBuilder;
import org.nakedobjects.testing.TestSpecification;
import org.nakedobjects.testing.TestSystem;


public class JavaSpecificationTest extends TestCase {

    public static void main(final String[] args) {
        junit.textui.TestRunner.run(JavaSpecificationTest.class);
    }

    private TestSystem system;
    private ReflectionPeerBuilder builder = new DummyBuilder();

    protected void setUp() throws ClassNotFoundException {
        LogManager.getLoggerRepository().setThreshold(Level.OFF);

        system = new TestSystem();
        system.init();
        system.addSpecificationToLoader(new TestSpecification());
        system.addSpecificationToLoader(new TestSpecification(String.class.getName()));
        system.addSpecificationToLoader(new TestSpecification(Object.class.getName()));

    }

    protected void tearDown() throws Exception {
        system.shutdown();
    }
/*
    public void testAggregated() throws Exception {
        system.addSpecificationToLoader(new TestSpecification(Aggregated.class.getName()));

        AbstractSpecification spec = new JavaSpecification(TestObjectAsAggregated.class, new DummyBuilder(), new JavaReflector(),
                false);
        spec.introspect(builder);
        assertEquals(NakedObjectSpecification.AGGREGATED, spec.getFeatures());
    }
*/
    public void testNoFlag() throws Exception {
        AbstractSpecification spec = new JavaSpecification(TestObject.class, new DummyBuilder(), new JavaReflector());
        spec.introspect(builder);
        assertEquals(0, spec.getFeatures());
    }

    public void testNotPersistable() throws Exception {
        system.addSpecificationToLoader(new TestSpecification(Interface1.class.getName()));
        system.addSpecificationToLoader(new TestSpecification(Interface2.class.getName()));
        system.addSpecificationToLoader(new TestSpecification(NonPersistable.class.getName()));

        AbstractSpecification spec = new JavaSpecification(JavaObjectForReflectorTransient.class, new DummyBuilder(),
                new JavaReflector());
        spec.introspect(builder);
        assertEquals(Persistable.TRANSIENT, spec.persistable());

    }

    public void testPersistable() throws Exception {
        system.addSpecificationToLoader(new TestSpecification(Interface1.class.getName()));
        system.addSpecificationToLoader(new TestSpecification(Interface2.class.getName()));
        system.addSpecificationToLoader(new TestSpecification(int.class.getName()));
        
        AbstractSpecification spec = new JavaSpecification(JavaObjectForReflector.class, new DummyBuilder(), new JavaReflector());
        spec.introspect(builder);
        assertEquals(Persistable.USER_PERSISTABLE, spec.persistable());
    }

    public void testService() throws Exception {
        AbstractSpecification spec = new JavaSpecification(TestObjectAsService.class, new DummyBuilder(), new JavaReflector());
        spec.introspect(builder);
        spec.markAsService();
        assertEquals(NakedObjectSpecification.SERVICE, spec.getFeatures());
    }
}
// Copyright (c) Naked Objects Group Ltd.
