package org.nakedobjects.nof.reflect.java.reflect;

import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.log4j.Level;
import org.apache.log4j.LogManager;
import org.nakedobjects.noa.annotations.MemberOrderAnnotation;


public class MemberOrderComparatorTest extends TestCase {

    public static void main(final String[] args) {
        junit.textui.TestRunner.run(new TestSuite(MemberOrderComparatorTest.class));
    }


    private MemberOrderComparator comparator, laxComparator;
    
    private MemberPeerStub m1 = new MemberPeerStub("abc");
    private MemberPeerStub m2 = new MemberPeerStub("abc");
    
    protected void setUp() {
        LogManager.getLoggerRepository().setThreshold(Level.OFF);
        
        comparator = new MemberOrderComparator(true);
        laxComparator = new MemberOrderComparator(false);
    }

    protected void tearDown() throws Exception {
    }

    public void testDefaultGroupOneComponent() {
        m1.addExtension(new MemberOrderAnnotation("","1"));
        m2.addExtension(new MemberOrderAnnotation("","2"));
        assertEquals(-1, comparator.compare(m1, m2));
    }

    public void testDefaultGroupOneComponentOtherWay() {
        m1.addExtension(new MemberOrderAnnotation("","2"));
        m2.addExtension(new MemberOrderAnnotation("","1"));
        assertEquals(+1, comparator.compare(m1, m2));
    }

    public void testDefaultGroupOneComponentSame() {
        m1.addExtension(new MemberOrderAnnotation("","1"));
        m2.addExtension(new MemberOrderAnnotation("","1"));
        assertEquals(0, comparator.compare(m1, m2));
    }

    public void testDefaultGroupOneSideRunsOutOfComponentsFirst() {
        m1.addExtension(new MemberOrderAnnotation("","1"));
        m2.addExtension(new MemberOrderAnnotation("","1.1"));
        assertEquals(-1, comparator.compare(m1, m2));
    }

    public void testDefaultGroupOneSideRunsOutOfComponentsFirstOtherWay() {
        m1.addExtension(new MemberOrderAnnotation("","1.1"));
        m2.addExtension(new MemberOrderAnnotation("","1"));
        assertEquals(+1, comparator.compare(m1, m2));
    }

    public void testDefaultGroupOneSideRunsTwoComponents() {
        m1.addExtension(new MemberOrderAnnotation("","1.1"));
        m2.addExtension(new MemberOrderAnnotation("","1.2"));
        assertEquals(-1, comparator.compare(m1, m2));
    }

    public void testDefaultGroupOneSideRunsTwoComponentsOtherWay() {
        m1.addExtension(new MemberOrderAnnotation("","1.2"));
        m2.addExtension(new MemberOrderAnnotation("","1.1"));
        assertEquals(+1, comparator.compare(m1, m2));
    }

    public void testDefaultGroupOneSideRunsLotsOfComponents() {
        m1.addExtension(new MemberOrderAnnotation("","1.2.5.8.3.3"));
        m2.addExtension(new MemberOrderAnnotation("","1.2.5.8.3.4"));
        assertEquals(-1, comparator.compare(m1, m2));
    }

    public void testDefaultGroupOneSideRunsLotsOfComponentsOtherWay() {
        m1.addExtension(new MemberOrderAnnotation("","1.2.5.8.3.4"));
        m2.addExtension(new MemberOrderAnnotation("","1.2.5.8.3.3"));
        assertEquals(+1, comparator.compare(m1, m2));
    }

    public void testDefaultGroupOneSideRunsLotsOfComponentsSame() {
        m1.addExtension(new MemberOrderAnnotation("","1.2.5.8.3.3"));
        m2.addExtension(new MemberOrderAnnotation("","1.2.5.8.3.3"));
        assertEquals(0, comparator.compare(m1, m2));
    }

    public void testNamedGroupOneSideRunsLotsOfComponents() {
        m1.addExtension(new MemberOrderAnnotation("abc","1.2.5.8.3.3"));
        m2.addExtension(new MemberOrderAnnotation("abc","1.2.5.8.3.4"));
        assertEquals(-1, comparator.compare(m1, m2));
    }

    public void testEnsuresInSameGroup() {
        m1.addExtension(new MemberOrderAnnotation("abc","1"));
        m2.addExtension(new MemberOrderAnnotation("def","2"));
        try {
            assertEquals(-1, comparator.compare(m1, m2));
            fail("Exception should have been thrown");
        } catch(IllegalArgumentException ex) {
            // expected
        }
    }

    public void testEnsuresInSameGroupCanBeDisabled() {
        m1.addExtension(new MemberOrderAnnotation("abc","1"));
        m2.addExtension(new MemberOrderAnnotation("def","2"));
        assertEquals(-1, laxComparator.compare(m1, m2));
    }

    public void testNonAnnotatedAfterAnnotated() {
        // don't annotate m1
        m2.addExtension(new MemberOrderAnnotation("def","2"));
        assertEquals(+1, comparator.compare(m1, m2));
    }

}
// Copyright (c) Naked Objects Group Ltd.
