package org.nakedobjects.nof.reflect.java.reflect;

import java.lang.reflect.Method;
import java.util.Date;

import junit.framework.TestCase;

public class MethodHelperTest extends TestCase {

    public void testStaticMethodWhenNullTarget() throws Exception {
        Class cls = TestClass.class;
        Method method = cls.getMethod("staticMethod", new Class[0]);
        MethodHelper helper = new MethodHelper(method);
        assertEquals("abcd", helper.execute(new TestClass()));
    }

    public void testInstanceMethod() throws Exception {
        Class cls = TestClass.class;
        Method method = cls.getMethod("instanceMethod", new Class[0]);
        MethodHelper helper = new MethodHelper(method);
        assertEquals("efgh", helper.execute(new TestClass()));
    }
    
    public void testCorrectNumberOfParametersUsed() throws Exception {
        Class cls = TestClass.class;
        Method method = cls.getMethod("instanceMethod", new Class[] {String.class, Date.class} );
        MethodHelper helper = new MethodHelper(method);
        assertEquals("jklm", helper.execute(new TestClass()));
    }
    
    public void testInstanceMethodCalledInSubclass() throws Exception {
        Class cls = ExtendedTestClass.class;
        Method method = cls.getMethod("instanceMethod", new Class[0]);
        MethodHelper helper = new MethodHelper(method);
        assertEquals("efgh", helper.execute(new ExtendedTestClass()));
    }

}

class TestClass {
    
    public static String staticMethod() {
        return "abcd";
    }
    
    public String instanceMethod() {
        return "efgh";
    }
    
    public String instanceMethod(String a, Date b) {
        return "jklm";
    }
}

class ExtendedTestClass extends TestClass {
    
}


// Copyright (c) Naked Objects Group Ltd.
