package org.nakedobjects.nof.reflect.java.value;

import org.nakedobjects.applib.value.Date;
import org.nakedobjects.applib.value.TestClock;
import org.nakedobjects.nof.core.adapter.value.AbstractValueAdapterTestCase;
import org.nakedobjects.nof.core.adapter.value.EncodingException;


public class DateAdapterTest extends AbstractValueAdapterTestCase {
 
    private DateAdapter adapter;

    protected void setUp() throws Exception {
        super.setUp();
        TestClock.initialize();
        
        super.value = adapter = new DateAdapter(new Date(2001, 2, 4));
    }

    public void testDateAsEncodedString() throws Exception {
        DateAdapter adapter = new DateAdapter(new Date(2005, 8, 13));
        assertEquals("20050813", new String(adapter.asEncodedString()));

        adapter = new DateAdapter(new Date(1999, 1, 1));
        assertEquals("19990101", new String(adapter.asEncodedString()));
    }

    public void testEmptyAsEncodedString() throws Exception {
        DateAdapter adapter = new DateAdapter();
        assertEquals("NULL", new String(adapter.asEncodedString()));
    }

    public void testNewAdapterIsEmpty() throws Exception {
        DateAdapter adapter = new DateAdapter();
        assertTrue(adapter.isEmpty());
    }

    public void testNewAdapterIsNotEmpty() throws Exception {
        DateAdapter adapter = new DateAdapter(new Date());
        assertFalse(adapter.isEmpty());
    }

    public void testParseEntryOfDaysAfterDate() throws Exception {
       
        adapter.parseTextEntry("+7");
        assertEquals(new Date(2001, 2, 11), adapter.getObject());
    }

    public void testParseEntryOfDaysAfterToToday() throws Exception {
        DateAdapter adapter = new DateAdapter();
        adapter.parseTextEntry("+5");
        assertEquals(new Date(2003, 8, 22), adapter.getObject());
    }

    public void testParseEntryOfDaysBeforeDate() throws Exception {
        adapter.parseTextEntry("-7");
        assertEquals(new Date(2001, 1, 28), adapter.getObject());
    }

    public void testParseEntryOfDaysBeforeToToday() throws Exception {
        DateAdapter adapter = new DateAdapter();
        adapter.parseTextEntry("-5");
        assertEquals(new Date(2003, 8, 12), adapter.getObject());
    }

    public void testParseEntryOfKeywordToday() throws Exception {
        DateAdapter adapter = new DateAdapter();
        adapter.parseTextEntry("today");
        assertEquals(new Date(2003, 8, 17), adapter.getObject());
    }

    public void testParseEntryOfWeeksAfterDate() throws Exception {
        adapter.parseTextEntry("+3w");
        assertEquals(new Date(2001, 2, 25), adapter.getObject());
    }

    public void testParseEntryOfWeeksAfterToToday() throws Exception {
        DateAdapter adapter = new DateAdapter();
        adapter.parseTextEntry("+4w");
        assertEquals(new Date(2003, 9, 14), adapter.getObject());
    }

    public void testParseEntryOfWeeksBeforeDate() throws Exception {
        adapter.parseTextEntry("-3w");
        assertEquals(new Date(2001, 1, 14), adapter.getObject());
    }

    public void testParseEntryOfWeeksBeforeToToday() throws Exception {
        DateAdapter adapter = new DateAdapter();
        adapter.parseTextEntry("-4w");
        assertEquals(new Date(2003, 7, 20), adapter.getObject());
    }
    

    public void testParseEntryOfMonthsAfterDate() throws Exception {
        adapter.parseTextEntry("+3m");
        assertEquals(new Date(2001, 5, 4), adapter.getObject());
    }

    public void testParseEntryOfMonthsAfterToToday() throws Exception {
        DateAdapter adapter = new DateAdapter();
        adapter.parseTextEntry("+4m");
        assertEquals(new Date(2003, 12, 17), adapter.getObject());
    }

    public void testParseEntryOfMonthsBeforeDate() throws Exception {
        adapter.parseTextEntry("-3m");
        assertEquals(new Date(2000, 11, 4), adapter.getObject());
    }

    public void testParseEntryOfMonthsBeforeToToday() throws Exception {
        DateAdapter adapter = new DateAdapter();
        adapter.parseTextEntry("-4m");
        assertEquals(new Date(2003, 4, 17), adapter.getObject());
    }

    public void testRestoreDate() throws Exception {
        DateAdapter adapter = new DateAdapter();
        adapter.restoreFromEncodedString("NULL");
        assertEquals(null, adapter.dateValue());
    }

    public void testRestoreOfInvalidDatal() throws Exception {
        DateAdapter adapter = new DateAdapter(new Date());
        try {
            adapter.restoreFromEncodedString("2003may12");
            fail();
        } catch (EncodingException expected) {}
    }

    public void testRestoreOfNull() throws Exception {
        DateAdapter adapter = new DateAdapter(new Date());
        adapter.restoreFromEncodedString("NULL");
        assertTrue(adapter.isEmpty());
    }

}

// Copyright (c) Naked Objects Group Ltd.
