package org.nakedobjects.nof.reflect.java.value;

import org.nakedobjects.applib.value.DateTime;
import org.nakedobjects.applib.value.TestClock;
import org.nakedobjects.nof.core.adapter.value.AbstractValueAdapterTestCase;

public class DateTimeAdapterTest extends AbstractValueAdapterTestCase {

    private DateTimeAdapter adapter;

    protected void setUp() throws Exception {
        super.setUp();

        TestClock.initialize();
        super.value = adapter = new DateTimeAdapter(new DateTime());
    }

    public void testNow() {
        adapter.parseTextEntry("now");
        assertEquals("17-Aug-2003 21:30", adapter.titleString());
    }

    public void testToday() {
        adapter.parseTextEntry("today");
        assertEquals("17-Aug-2003 21:30", adapter.titleString());
    }

    public void testEntryWithShortFormat() {
        adapter.parseTextEntry("21/5/07 10:30");
        assertEquals("21-May-2007 10:30", adapter.titleString());
    }

    public void testEntryWithMediumFormat() {
        adapter.parseTextEntry("21-May-2007 10:30");
        assertEquals("21-May-2007 10:30", adapter.titleString());
    }
    
    public void testEntryWithShortISOFormat() {
        adapter.parseTextEntry("20070521T1030");
        assertEquals("21-May-2007 10:30", adapter.titleString());
    }
        
    public void testEntryWithLongISOFormat() {
        adapter.parseTextEntry("2007-05-21 10:30");
        assertEquals("21-May-2007 10:30", adapter.titleString());
    }    

    public void testEmptyClears() {
        adapter.parseTextEntry("");
        assertEquals("", adapter.titleString());
    }

    public void testAddDayAndMonth() {
        adapter.parseTextEntry("+1d 1m");
        assertEquals("18-Sep-2003 21:30", adapter.titleString());
    }
    
    public void testSubtractDayAndMonth() {
        adapter.parseTextEntry("-1d 1m");
        assertEquals("16-Jul-2003 21:30", adapter.titleString());
    }
    
    public void testAddOneDay() {
        adapter.parseTextEntry("+1");
        assertEquals("18-Aug-2003 21:30", adapter.titleString());
    }

    public void testSubtractOneDay() {
        adapter.parseTextEntry("-1");
        assertEquals("16-Aug-2003 21:30", adapter.titleString());
    }

    public void testAddOneMonth() {
        adapter.parseTextEntry("+1m");
        assertEquals("17-Sep-2003 21:30", adapter.titleString());
    }

    public void testAddOneYear() {
        adapter.parseTextEntry("+1y");
        assertEquals("17-Aug-2004 21:30", adapter.titleString());
    }
    

    public void testAddOneHour() {
        adapter.parseTextEntry("+1H");
        assertEquals("17-Aug-2003 22:30", adapter.titleString());
    }


    public void testAddOneMinute() {
        adapter.parseTextEntry("+1H");
        assertEquals("17-Aug-2003 22:30", adapter.titleString());
    }

}


// Copyright (c) Naked Objects Group Ltd.
