package org.nakedobjects.nof.reflect.java.value;

import junit.framework.TestCase;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.nakedobjects.applib.value.TestClock;
import org.nakedobjects.applib.value.Time;
import org.nakedobjects.nof.core.adapter.value.AbstractValueAdapterTestCase;
import org.nakedobjects.nof.core.adapter.value.EncodingException;
import org.nakedobjects.testing.TestSystem;


public class TimeAdapterTest extends AbstractValueAdapterTestCase {

    private TimeAdapter adapter;

    protected void setUp() throws Exception {
        Logger.getRootLogger().setLevel(Level.OFF);
        new TestSystem().init();
        TestClock.initialize();
        super.value = adapter = new TimeAdapter(new Time(8, 13));
    }

    public void testTimeAsEncodedString() throws Exception {
        assertEquals("081300000", new String(adapter.asEncodedString()));

        adapter = new TimeAdapter(new Time(1, 1));
        assertEquals("010100000", new String(adapter.asEncodedString()));
    }

    public void testEmptyAsEncodedString() throws Exception {
        TimeAdapter adapter = new TimeAdapter();
        assertEquals("NULL", new String(adapter.asEncodedString()));
    }

    public void testNewAdapterIsEmpty() throws Exception {
        TimeAdapter adapter = new TimeAdapter();
        assertTrue(adapter.isEmpty());
    }

    public void testNewAdapterIsNotEmpty() throws Exception {
        TimeAdapter adapter = new TimeAdapter(new Time());
        assertFalse(adapter.isEmpty());
    }

    public void testParseEntryOfHoursAfterTime() throws Exception {
        TimeAdapter adapter = new TimeAdapter(new Time(15, 10));
        adapter.parseTextEntry("+5H");
        assertEquals(new Time(20, 10), adapter.getObject());
    }

    public void testParseEntryOfHoursAfterNow() throws Exception {
        TimeAdapter adapter = new TimeAdapter();
        adapter.parseTextEntry("+5H");
        assertEquals(new Time(2, 30), adapter.getObject());
    }

    public void testParseEntryOfHoursBeforeTime() throws Exception {
        TimeAdapter adapter = new TimeAdapter(new Time(12, 4));
        adapter.parseTextEntry("-7H");
        assertEquals(new Time(5, 4), adapter.getObject());
    }

    public void testParseEntryOfHoursBeforeToNow() throws Exception {
        TimeAdapter adapter = new TimeAdapter();
        adapter.parseTextEntry("-5H");
        assertEquals(new Time(16, 30), adapter.getObject());
    }

    public void testParseEntryOfKeywordNow() throws Exception {
        TimeAdapter adapter = new TimeAdapter();
        adapter.parseTextEntry("now");
        assertEquals(new Time(), adapter.getObject());
    }

    public void testRestoreTime() throws Exception {
        TimeAdapter adapter = new TimeAdapter();
        adapter.restoreFromEncodedString("NULL");
        assertEquals(null, adapter.getObject());
    }

    public void testRestoreOfInvalidDatal() throws Exception {
        TimeAdapter adapter = new TimeAdapter(new Time());
        try {
            adapter.restoreFromEncodedString("two ten");
            fail();
        } catch (EncodingException expected) {}
    }

    public void testRestoreOfNull() throws Exception {
        TimeAdapter adapter = new TimeAdapter(new Time());
        adapter.restoreFromEncodedString("NULL");
        assertTrue(adapter.isEmpty());
    }

}

// Copyright (c) Naked Objects Group Ltd.
